package project;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import exceptions.CategoryInvalidException;
import exceptions.DateInvalidException;
import exceptions.DescriptionInvalidException;
import exceptions.ImageInvalidException;
import exceptions.ItemNonExistentException;
import exceptions.PriceInvalidException;
import exceptions.TitleInvalidException;

public class Seller extends User {

	private ArrayList<Auction> myAuctions;

	public Item addItem(String title, String description, ArrayList<ItemPicture> itemPictures, String category)
			throws CategoryInvalidException, TitleInvalidException, DescriptionInvalidException, ImageInvalidException {

		if (!title.matches("^[a-zA-Z]+$") || title.length() > 30 || title.length() < 1) {

			throw new TitleInvalidException();

		} else if (description.matches("^\\W+$|\\d+") || description.length() > 500 || description.length() < 1) {

			throw new DescriptionInvalidException();
		}

		else if (itemPictures.size() < 1 || itemPictures.size() > 10) {

			throw new ImageInvalidException();

		} else {

			Category c = Category.getCategory(category);

			if (c == null) {

				throw new CategoryInvalidException();

			} else {

				Item i = new Item(title, description, itemPictures, c);
				return i;

			}

		}

	}

	public static Seller getInstance() {

		Seller s = new Seller();
		return s;

	}

	public static boolean isValidFormat(String format, String value) {

		Date date = null;
		
		try {

			SimpleDateFormat sdf = new SimpleDateFormat(format);
			date = sdf.parse(value);

			if (value.equals(sdf.format(date))) {

				return true;

			} else {

				return false;

			}

		} catch (ParseException ex) {

			return false;

		}

	}

	public boolean createAuction(Item item, String s, String e, double price, double bidIncrement)
			throws PriceInvalidException, DateInvalidException, ItemNonExistentException {

		String format = "dd/MM/yyyy hh:mm:ss";		
		SimpleDateFormat sdf = new SimpleDateFormat(format);
		Date start, end;

		if (!isValidFormat(format, s) || !isValidFormat(format, e)) {

			throw new DateInvalidException();

		} else {

			try {

				start = sdf.parse(s);
				end = sdf.parse(e);

			} catch (ParseException p) {

				throw new DateInvalidException();

			}

			if (!Item.items.contains(item)) {

				throw new ItemNonExistentException();

			} else if (start.after(end)) {

				throw new DateInvalidException();

			} else if (price > 0) {

				Auction a = new Auction(item, start, end, price, bidIncrement);
				return true;

			} else {

				throw new PriceInvalidException();
			}

		}

	}

	public void setMyAuctions(ArrayList<Auction> sellerAuctions) {
		// TODO Auto-generated method stub

	}

	public ArrayList<Auction> displayMyAuctions() {
		// TODO Auto-generated method stub
		return null;
	}

}
