package unitTest;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.Random;
import java.util.function.Function;
import java.util.stream.Stream;

import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.api.function.Executable;
import org.junit.jupiter.api.function.ThrowingConsumer;

import exceptions.CategoryInvalidException;
import exceptions.DescriptionInvalidException;
import exceptions.TitleInvalidException;
import project.Category;
import project.Item;
import project.User;

public class ItemUnitTest {

	@TestFactory
	public Stream<DynamicTest> deleteTestTrue() {

		Iterator<Integer> inputGenerator = new Iterator<Integer>() {

			int itemId;
			Random random = new Random();

			@Override
			public boolean hasNext() {

				return random.nextInt(100) % 10 != 0;

			}

			@Override
			public Integer next() {

				itemId = (int) (random.nextDouble() * 10);
				Item i = new Item();
				i.setId(itemId);
				return itemId;
			}

		};

		Function<Integer, String> displayNameGenerator = (input) -> "item ID: " + input;

		ThrowingConsumer<Integer> testExecutor = (input) -> assertTrue(Item.delete(input));

		return DynamicTest.stream(inputGenerator, displayNameGenerator, testExecutor);

	}

	@TestFactory
	public Stream<DynamicTest> deleteTestFalse() {

		Iterator<Integer> inputGenerator = new Iterator<Integer>() {

			int itemId;
			Random random = new Random();

			@Override
			public boolean hasNext() {

				return random.nextInt(100) % 10 != 0;

			}

			@Override
			public Integer next() {

				itemId = (int) (random.nextDouble() * 10);
				return itemId;
			}

		};

		Function<Integer, String> displayNameGenerator = (input) -> "item ID: " + input;

		ThrowingConsumer<Integer> testExecutor = (input) -> assertFalse(Item.delete(input));

		return DynamicTest.stream(inputGenerator, displayNameGenerator, testExecutor);

	}

	@TestFactory
	public Collection<DynamicTest> modifyTest() {

		ArrayList<String> title = new ArrayList<String>(
				Arrays.asList("", "9999", "t-#\\", "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"));
		ArrayList<String> description = new ArrayList<String>(Arrays.asList("", "9999",
				"aaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n" + "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
						+ "aaaaaaaaaaaaaaaa"));
		ArrayList<String> category = new ArrayList<String>(
				Arrays.asList("bike"));
		ArrayList<String> titleTestNames = new ArrayList<String>(Arrays.asList("title empty", "numerical title",
				"symbols present in the title", "title longer than 30 characters", "cateogry not correct"));
		ArrayList<String> descriptionTestNames = new ArrayList<String>(Arrays.asList("description empty", "numerical description",
				"title longer than 500 characters"));
		ArrayList<String> categoryTestNames = new ArrayList<String>(Arrays.asList("cateogry not correct"));
		ArrayList<Class<? extends Exception>> titleExceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(TitleInvalidException.class, TitleInvalidException.class, TitleInvalidException.class,
						TitleInvalidException.class));
		ArrayList<Class<? extends Exception>> descriptionExceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(DescriptionInvalidException.class, DescriptionInvalidException.class, DescriptionInvalidException.class));
		ArrayList<Class<? extends Exception>> categoryExceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(CategoryInvalidException.class));

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		Item item = Item.getInstance();
		item.setTitle("title");
		item.setDescription("description");
		item.setCategory(new Category("camera"));

		for (int i = 0; i < title.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(titleExceptions.get(j),
					() -> item.modify(title.get(j), "description", "camera"));
			DynamicTest dTest = DynamicTest.dynamicTest(titleTestNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		for (int i = 0; i < description.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(descriptionExceptions.get(j),
					() -> item.modify("title", description.get(j), "camera"));
			DynamicTest dTest = DynamicTest.dynamicTest(descriptionTestNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		for (int i = 0; i < category.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(categoryExceptions.get(j),
					() -> item.modify("title", "description", category.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest(categoryTestNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

}
